/*------------------------------------------------------------------------------
    File        : dictmain.p
    Purpose     : Manages all dictionary related tasks
    Description : Main Procedure for loading dictionary information 
                  into UML repository
    Author(s)   : Thomas Mercer-Hursh
    Created     : Sun Dec 16 16:58:21 PST 2007
    Notes       : Assumes Dictionary data source has been connected with alias "DictSource"
                  linTraceDetail determines level of lcgging information
                  1 = basic progress; 2 = table level; 3 = all details 
  ----------------------------------------------------------------------------*/
using com.cintegrity.Lib.Exception.*.
/* ***************************  Definitions  ******************************** */
                                                        /* Repository Buffers */
define buffer lbfPackage for Repository.t_Package.
define buffer lbfParentPackage for Repository.t_Package.
                                                        /* Dictionary Buffers */
define buffer lbfDB for DictSource._DB.
define buffer lbfFile for DictSource._File.
define buffer lbfField for DictSource._Field.
define buffer lbfIndex for DictSource._Index.
define buffer lbfIndexField for DictSource._Index-Field.
define buffer lbfStorageObject for DictSource._StorageObject.
define buffer lbfArea for DictSource._Area.
define buffer lbfSequence for DictSource._Sequence.
                                                               /* Current IDs */
define variable linDataModelPackageID as integer  no-undo.
define variable linDatabasePackageID as integer  no-undo.
define variable linPackageID as integer no-undo.
define variable lchPackageName as character no-undo.
define variable linObjectID as integer no-undo.
define variable linAttributeID as integer  no-undo.
define variable linOperationID as integer no-undo.
define variable linObjectPropertyID as integer  no-undo.
define variable linAttributeTagID as integer  no-undo.
define variable linOperationTagID as integer  no-undo.
                                                            /* Work Variables */
define variable lchDictSourceName as character no-undo.
define variable linNotNull as integer  no-undo.
define variable lchIndexType as character no-undo.
define variable lchIndexName as character no-undo.
define variable lriDB as recid no-undo.
define variable lchIndexFieldStereotype as character no-undo.
define variable linTraceDetail as integer no-undo initial 1.
define variable lchIsWordIndex as character no-undo.
                                                           /* Class Variables */
define variable lobExceptionStack as class SimpleExceptionStack no-undo.
                                                               /* Temp Tables */
define temp-table ttTablePackage no-undo
    field chPackageName as character
    field inPackageID as integer
    index ixByName as primary unique chPackageName
    .
/* *************************  Initialization ******************************** */
lobExceptionStack = new SimpleExceptionStack().
if not connected( "DictSource" )
then do:
  lobExceptionStack:pushException( 201, 1, "Dictionary Data Source must be connected with alias `DictSource'" ).
  run CleanUp.
  return.
end.
else do:
  lchDictSourceName = pdbname("DictSource").
end.
lobExceptionStack:trace( "Running Dictionary to UML routine at " + string(now) ).
/* ***************************  Main Block  ********************************* */
                                              /* Get Recid(_DB) for reference */
do for lbfDB:
  find first lbfDB no-lock.
  lriDB = recid(lbfDB).
end.                                        
                                      /* find datamodel package in repository */
run spFindDataModelPackage (
    output linDataModelPackageID
    ).
if lobExceptionStack:numExceptions( 1 ) gt 0
    or linDataModelPackageID eq ?
then do:
  lobExceptionStack:pushException( 202, 1, "Fatal Error in com.cintegrity.ADBtoUML.dict.dictmain.p - aborting" ).
  run CleanUp.
  return.
end. 
                                                   /* CREATE oeDatabase entry */
run spCreatePackage (
    input lchDictSourceName,
    input linDataModelPackageID,
    output linDatabasePackageID
    ). 
if     linDatabasePackageID eq ?
    or linDatabasePackageID eq 0
then do:
  lobExceptionStack:pushException( 223, 2, "Failure to Create Package for " + lchDictSourceName ).
end.    
                                               /* Create corresponding object */
run spCreateObject (
    input "Package",
    input lchDictSourceName,
    input linDatabasePackageID,
    input 0,
    input "oeDatabase",
    input "",
    input "",
    output linObjectID
    ).
/* -------------------------------------------------------------------------- */
                                                                /* Add Tables */ 
for each  lbfFile no-lock
    where lbfFile._File-num lt 32768
    by lbfFile._File-num
    :
                                                  /* Assign package for table */
  run spGetTablePackage (
      input lbfFile._File-Name,
      output lchPackageName
      ).
  
  if linTraceDetail ge 2
  then lobExceptionStack:trace( "Processing table " + lbfFile._File-Name + " in package " + lchPackageName ).
                                      /* Find/Create TT for Package for Table */
  find ttTablePackage
      where ttTablePackage.chPackage = lchPackageName no-error.
  if not available( ttTablePackage )
  then do:
    run spCreatePackage (
        input lchPackageName,
        input linDatabasePackageID,
        output linPackageID
        ).
    if     linPackageID eq ?
        or linPackageID eq 0
    then do:
      lobExceptionStack:pushException( 221, 2, "Failure to Create Package for " + lchPackageName ).
    end.
    create ttTablePackage.
    assign
        ttTablePackage.chPackageName = lchPackageName
        ttTablePackage.inPackageID = linPackageID
        .
                                               /* Create corresponding object */
    run spCreateObject (
        input "Package",
        input lchPackageName,
        input linPackageID,
        input 0,
        input "",
        input "",
        input "",
        output linObjectID
        ).
    if     linObjectID eq ?
        or linObjectID eq 0
    then do:
      lobExceptionStack:pushException( 222, 2, "Failure to Create Object for Package " + lchPackageName ).
    end.        
  end.  
                                         /* Create entry for table in package */
  run spCreateObject (
      input "Class",
      input lbfFile._File-name,
      input ttTablePackage.inPackageID,
      input 0,
      input "table",                                     /* Should be oeTable */
      input lbfFile._Desc,
      input "",
      output linObjectID
      ).
    if     linObjectID eq ?
        or linObjectID eq 0
    then do:
      lobExceptionStack:pushException( 222, 2, "Failure to Create Object for Table " + lbfFile._File-name ).
    end.   
                                                        /* Add tags for table */
  { com/cintegrity/ABLtoUML/dict/tabletags.i }
/* -------------------------------------------------------------------------- */
                                                            /* Create Columns */
  for each lbfField no-lock
      where lbfField._File-Recid eq recid(lbfFile)
      by lbfField._Order
      :
    if linTraceDetail ge 3
    then lobExceptionStack:trace( "Processing table " + lbfFile._File-Name + " and column " + lbfField._Field-nam ).

    if lbfField._Mandatory
    then linNotNull = 1.
    else linNotNull = 0.
      
    run spCreateAttribute (
        input linObjectID,
        input lbfField._Field-name,
        input "column",    /* Should be oeColumn */
        input lbfField._Desc,
        input lbfField._Order,
        input linNotNull,
        input lbfField._Initial,
        input lbfField._Data-type,
        input "AttribTable",
        output linAttributeID
        ).
                                                      /* Add tags for columns */
    { com/cintegrity/ABLtoUML/dict/columntags.i }  
  end.                                                
/* -------------------------------------------------------------------------- */      
                                                            /* Create Indexes */
  for each lbfIndex no-lock
      where recid(lbfFile) eq lbfIndex._File-Recid
      :
    if lbfFile._Prime-Index eq recid( lbfIndex )
    then lchIndexType = "PK".
    else lchIndexType = "index".
    lchIndexName = lchIndexType + "_" + lbfIndex._Index-Name.

    if linTraceDetail ge 3
    then lobExceptionStack:trace( "Processing table " + lbfFile._File-Name + " and index " + lchIndexName ).
          
    run spCreateOperation (
        input linObjectID,
        input lchIndexName,
        input lchIndexType,
        output linOperationID 
        ).
    if lbfIndex._Unique
    then do:
      run spCreateOperationPost (
          input linOperationID,
          input "IsUnique", 
          input "Functional"
          ).
    end.
    
    if lbfIndex._WordIdx eq ?
    then lchIsWordIndex = "No".
    else lchIsWordIndex = "Yes".
        run spCreateOperationTag (
        input linOperationID,
        input "IsWordIndex", 
        input lchIsWordIndex,
        output linOperationTagID
        ).
                                                   /* Create Index Components */
    for each lbfIndexField no-lock
        where recid(lbfIndex) eq lbfIndexField._Index-Recid
        :
      find lbfField no-lock where lbfIndexField._Field-Recid = recid(lbfField).
      if lbfIndexField._Ascending
      then do:
        if lbfIndexField._Abbreviate
        then lchIndexFieldStereotype = "AscAbv".
        else lchIndexFieldStereotype = "Asc".
      end.
      else do:
        if lbfIndexField._Abbreviate
        then lchIndexFieldStereotype = "DescAbv".
        else lchIndexFieldStereotype = "Desc".
      end.
      run spCreateOperationParam (
          input linOperationID,
          input lbfField._Field-Name,
          input lbfField._Data-Type,
          input lbfField._Initial,
          input lbfIndexField._Index-Seq - 1,
          input "in",
          input lchIndexFieldStereotype
          ).
    end.
  end.
end.
/* -------------------------------------------------------------------------- */
                                                         /* Process sequences */
lobExceptionStack:trace( "Processing sequences" ).
run spCreatePackage (
    input "Sequences",
    input linDatabasePackageID,
    output linPackageID
    ).
run spCreateObject (
    input "Package",
    input "Sequences",
    input linPackageID,
    input 0,
    input "",
    input "",
    input "",
    output linObjectID
    ).
for each lbfSequence no-lock:
  
  if linTraceDetail ge 2
  then lobExceptionStack:trace( "Processing sequence " + lbfSequence._Seq-Name ).
  
  run spCreateObject(
      input "Class",
      input lbfSequence._Seq-Name,
      input linPackageID,
      input 0,
      input "oeSequence",
      input "",
      input "",
      output linObjectID
      ).
  run spCreateAttribute (
      input linObjectID,
      input lbfSequence._Seq-Name,
      input "",                                                /* Should be ? */
      input "",
      input 1,
      input 1,
      input lbfSequence._Seq-Init,
      input "Integer",
      input "",
      output linAttributeID
      ).
                                                    /* Add tags for sequences */
    { com/cintegrity/ABLtoUML/dict/seqtags.i }  
end.

                                                       /* Export log thus far */
lobExceptionStack:writeTextExternal( 9, "ABLtoUMLLog.txt", true ).
lobExceptionStack:clearExceptions().

run CleanUp.
return.
/* ****************************  Clean Up  ********************************** */
procedure CleanUp:
  delete object lobExceptionStack.
  lobExceptionStack = ?.
  empty temp-table ttTablePackage.
end.
/* *************************** End of Program ******************************* */